@@ -1,6 +1,7 @@
 package tokenclone
 
 import (
+	"encoding/json"
 	"os"
 	"regexp"
 	"strconv"
@@ -12,13 +13,13 @@ import (
 	"github.com/hashicorp/consul/sdk/testutil"
 	"github.com/hashicorp/consul/testrpc"
 	"github.com/mitchellh/cli"
+	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
 
 func parseCloneOutput(t *testing.T, output string) *api.ACLToken {
 	// This will only work for non-legacy tokens
-	re := regexp.MustCompile("Token cloned successfully.\n" +
-		"AccessorID:       ([a-zA-Z0-9\\-]{36})\n" +
+	re := regexp.MustCompile("AccessorID:       ([a-zA-Z0-9\\-]{36})\n" +
 		"SecretID:         ([a-zA-Z0-9\\-]{36})\n" +
 		"(?:Namespace:        default\n)?" +
 		"Description:      ([^\n]*)\n" +
@@ -58,7 +59,7 @@ func TestTokenCloneCommand_noTabs(t *testing.T) {
 	}
 }
 
-func TestTokenCloneCommand(t *testing.T) {
+func TestTokenCloneCommand_Pretty(t *testing.T) {
 	t.Parallel()
 	req := require.New(t)
 
@@ -164,3 +165,84 @@ func TestTokenCloneCommand(t *testing.T) {
 		req.Equal(cloned.Policies, apiToken.Policies)
 	})
 }
+
+func TestTokenCloneCommand_JSON(t *testing.T) {
+	t.Parallel()
+	req := require.New(t)
+
+	testDir := testutil.TempDir(t, "acl")
+	defer os.RemoveAll(testDir)
+
+	a := agent.NewTestAgent(t, t.Name(), `
+   primary_datacenter = "dc1"
+   acl {
+      enabled = true
+      tokens {
+         master = "root"
+      }
+   }`)
+
+	defer a.Shutdown()
+	testrpc.WaitForLeader(t, a.RPC, "dc1")
+
+	// Create a policy
+	client := a.Client()
+
+	_, _, err := client.ACL().PolicyCreate(
+		&api.ACLPolicy{Name: "test-policy"},
+		&api.WriteOptions{Token: "root"},
+	)
+	req.NoError(err)
+
+	// create a token
+	token, _, err := client.ACL().TokenCreate(
+		&api.ACLToken{Description: "test", Policies: []*api.ACLTokenPolicyLink{&api.ACLTokenPolicyLink{Name: "test-policy"}}},
+		&api.WriteOptions{Token: "root"},
+	)
+	req.NoError(err)
+
+	// clone with description
+	t.Run("Description", func(t *testing.T) {
+		ui := cli.NewMockUi()
+		cmd := New(ui)
+
+		args := []string{
+			"-http-addr=" + a.HTTPAddr(),
+			"-id=" + token.AccessorID,
+			"-token=root",
+			"-description=test cloned",
+			"-format=json",
+		}
+
+		code := cmd.Run(args)
+		req.Empty(ui.ErrorWriter.String())
+		req.Equal(code, 0)
+
+		output := ui.OutputWriter.String()
+		var jsonOutput json.RawMessage
+		err = json.Unmarshal([]byte(output), &jsonOutput)
+		assert.NoError(t, err)
+	})
+
+	// clone without description
+	t.Run("Without Description", func(t *testing.T) {
+		ui := cli.NewMockUi()
+		cmd := New(ui)
+
+		args := []string{
+			"-http-addr=" + a.HTTPAddr(),
+			"-id=" + token.AccessorID,
+			"-token=root",
+			"-format=json",
+		}
+
+		code := cmd.Run(args)
+		req.Empty(ui.ErrorWriter.String())
+		req.Equal(code, 0)
+
+		output := ui.OutputWriter.String()
+		var jsonOutput json.RawMessage
+		err = json.Unmarshal([]byte(output), &jsonOutput)
+		assert.NoError(t, err)
+	})
+}
