@@ -1,5 +1,4 @@
 import Serializer from './application';
-import { get } from '@ember/object';
 
 export default Serializer.extend({
   primaryKey: 'Name',
@@ -8,7 +7,7 @@ export default Serializer.extend({
       case 'findAll':
         return payload.map(item => {
           return {
-            [get(this, 'primaryKey')]: item,
+            [this.primaryKey]: item,
           };
         });
     }
